@@ -7,7 +7,7 @@ import sys
 from git import GitError
 
 from FeatureCloud.api.imp.exceptions import FCException
-from FeatureCloud.api.imp.util import getcwd_fslash, get_docker_client
+from FeatureCloud.api.imp.util import getcwd_fslash, get_docker_client, remove_dir
 import pydot
 import importlib
 
@@ -43,12 +43,20 @@ def new(name: str, directory: str = '.', template_name: str = 'app-blank') -> st
 
     try:
         app_path = os.path.join(directory, name.lower())
-        repo = git.Repo.clone_from(create_link(template_name), app_path)
+        # depth=1 clones with minimal history information, we delete the .git folder anyway
+        repo = git.Repo.clone_from(create_link(template_name), app_path, multi_options=['--depth=1'])
         repo.delete_remote('origin')
-        return app_path
     except GitError as e:
         raise FCException(e)
 
+    # delete the .git folder
+    try:
+        remove_dir(os.path.join(app_path, '.git'))
+    except:
+        pass
+
+    return app_path
+
 
 def build(path: str = ".", image_name: str = None, tag: str = "latest", rm: str = True):
     """ Build app image.
@@ -248,7 +256,9 @@ def plot_state_diagram(path: str, package: str, states: str, plot_name: str):
 
     for t in app.transitions:
         transition = app.transitions[t]
-        state_edge = pydot.Edge(transition[0].name, transition[1].name, label=t)
+        label = t if transition[4] is None else transition[4]
+
+        state_edge = pydot.Edge(transition[0].name, transition[1].name, label=label)
         if transition[2] and transition[3]:
             state_edge.set('color', 'purple')
         elif transition[3]:
